/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;

public class BlackWhitePresentation implements PresentationType {

    private final PresentationType pt = this;

    private final class MyPresentation extends ColorModel implements Presentation {

	private final Picture p;
	private Image img = null;

	public MyPresentation(Picture p) {super(31); this.p = p;}

	// ColorModel
	public int getRed(int pixel) {return (pixel == 1) ? 255 : 0;}
	public int getGreen(int pixel) {return (pixel == 1) ? 255 : 0;}
	public int getBlue(int pixel) {return (pixel == 1) ? 255 : 0;}
	public int getAlpha(int pixel) {return 255;}

	// Presentation
	public Image getImage() throws PictureException {

	    if (img != null) return img;

	    if (p.getNumberOfLayers() != 1) 
		throw new PictureException("Ungltigs Bild");

	    Layer layer = p.getLayer(0);
	    double max = layer.getMaximum();	    double min = layer.getMinimum();
	    double gr = (max + min) / 2;
	    int width = p.getWidth(); int height = p.getHeight();
	    int[] id = new int[width * height];

	    for (int x = 0; x < width; x++)
		for (int y = 0; y < height; y++) 
		    id[y * width + x] = (layer.getValue(x, y) > gr) ? 1 : 0;
	    
	    
	    MemoryImageSource mis = new MemoryImageSource(width, height, this, id, 0, width);
	    img = Toolkit.getDefaultToolkit().createImage(mis);
	    return img;
	}

	public JPopupMenu getJPopupMenu(ActionListener e) {

	    return PresentationFactory.getJPopupMenu(p, e);
	}

	public PresentationType getPresentationType() {return pt;}

	public Presentation getCopy(Picture p) {

	    if (!isCompatible(p))
		return PresentationFactory.createDefaultPresentation(p);
	    
	    return new MyPresentation(p);
	}

	public String toString() {

	    return getName() + "$";
	}
    }

    public BlackWhitePresentation() {}

    public boolean isCompatible(Picture mlp) {

	return mlp.getNumberOfLayers() == 1;
    }

    public String getName() {return "Schwarz/Wei-Darstellung";}

    public Presentation createPresentation(Picture p) throws PictureException {

	if (!isCompatible(p))
	    throw new PictureException("Das Bild hat zu viele Ebenen");

	return new MyPresentation(p);
    }

    public Presentation createPresentation(Picture p, String s) throws PictureException {

	return createPresentation(p);
    }
}
