/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;
import picture.filter.*;

/**
 * Implementiert eine Korellation zwischen zwei Layer-Objekten. Beispiel:
 * Seien <tt>a</tt> und <tt>b</tt> Layer-Objekte. Gesucht ist dann die
 * Korellation von <tt>a</tt> und <tt>b</tt>. Dies ist dann mit folgender
 * Zeile realisierbar:
 * <pre>
 *   Layer aKorrB = (new Correlation(a)).of(b);
 * </pre>
 * 
 * @version 1.0
 */
public class Correlation extends Combination {

    private static final FFT fft = new FFT(false, false); 
    private static final FFT fftI = new FFT(true, false);
    private static final FundamentalOperation mal = 
	new FundamentalOperation(FundamentalOperation.COMPLEX_PRODUCT);

    public Correlation() {}

    public Layer of(ProgressHandler ph, Layer a, Layer b) throws PictureException {

	throw new PictureException("Veraltete Methode:\n" +
				   "Layer picture.Correlation.of(Layer, Layer)");
    }

    /**
     * Erzeugt die Korellation von <tt>a</tt> und <tt>b</tt>. 
     * <p>
     * Diese Methode unterstuetzt den Progress-Listener-Mechanismus der Klasse
     * Layer und wirft <tt>6 * a.width</tt> ProgressEvents aus.
     * 
     * @param a: erster Operand
     * @param b: zweiter Operand
     * @return <tt>a</tt> korreliert mit <tt>b</tt>.
     */
    public Picture of(Picture a, Picture b) 
	throws PictureException {

	if (a.getNumberOfLayers() == 1) a = realToComplex(a);
	if (b.getNumberOfLayers() == 1) b = realToComplex(b);

	if (!a.isComplex() || !b.isComplex())
	    throw new PictureException("Es knnen nur reelle und komplexe Bilder\n" +
				       "miteinander korreliert werden.");

	Picture p = null;
	ProgressHandler ph = a.getProgressHandler();

	synchronized(b) {
	    ProgressHandler ph0 = b.getProgressHandler();
	    b.setProgressHandler(ph);
	    p = fft.of(b);
	    b.setProgressHandler(ph0);
	}

	Picture q = fftI.of(a);
	q.setProgressHandler(ph); q = mal.of(p, q); 
	q.setProgressHandler(ph); q = fftI.of(q);
	return q;
    }

    private Picture realToComplex(Picture p) {

	double[][] d = new double[p.getWidth()][p.getHeight()];

	Layer[] pics = new Layer[]{p.getLayer(0), new Layer(d)};

	return new Picture(p, pics) {
	    
	    public boolean isComplex() {return true;}
	};
    }
}
