/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Eine Presentation dient als Schnittstelle zu java.awt.*. Mit Hilfe einer Presentation
 * ist es m&ouml;glich, ein Picture-Objekt in ein java.awt.Image-Objekt umzuwandeln, das
 * auf einem Graphics angezeigt werden kann. Die Presentation kann von verschiedenen 
 * Parametern abh&auml;gnen, sie stellt ein JPopupMenu zur Verf&uuml;gung, &uuml;ber das
 * diese Einstellungen eingestellt werden k&ouml;nnen. 
 * <p>
 * Presentation lassen geh&ouml;ren zu verschiedenen PresentationTypes, die jeweils globale
 * Daten einer Familie von Presentations verwalten. Man kann sich die Dienste von
 * PresentationType wie statische Methoden einer Presentation vorstellen. (Implementierungstrick,
 * da man im interface statische Methoden nicht erzwingen kann)
 * <p>
 * Zur Verwaltung verschiedener PresentationTypes dient schlie&szlig;lich eine Klasse
 * PresentationFactory, die nur statische Methoden zur Verf&uuml;gung stellt und nicht
 * zum Initialisieren gedacht ist. Der Anwender m&ouml;ge nur auf die Dienste der
 * PresentationFactory zugreifen.
 *
 * @version 1.0
 * @author Manuel Kauers
 * @author Stefan R&ouml;hrich
 * @see PresentationType
 * @see PresentationFactory
 */
public interface Presentation {

    /**
     * Erzeugt die aktuelle Ansicht eines Pictures
     * 
     * @return aktuelle Ansicht <tt>mpl</tt>.
     * @exception PictureException: Falls was schief l&auml;ft.
     */
    public Image getImage() throws PictureException;

    /**
     * Erzeugt ein <tt>JPopupMenu</tt>, &uuml;ber das die Einstellungen dieses
     * Objektes ver&auml;ndert werden k&ouml;nnen. 
     *
     * Das Men&uuml; verf&uuml;gt auch &uuml;ber Eintr&auml;ge, mit deren Hilfe
     * die Presentation des Bildes gewechselt werden kann.
     *
     * @param al: Wird ein Men&uuml;eintrag bet&auml;tigt, wird auf <tt>al</tt> ein
     * ActionEvent ausgef&uuml;hrt.
     * @return Men&uuml; zu dieser Presentation.
     */
    public JPopupMenu getJPopupMenu(ActionListener al);

    /**
     * Liefert den PresentationType, der mit dieser Presentation assoziiert ist
     *
     * @return zugeh&ouml;riger PresentationType
     */
    public PresentationType getPresentationType();

    /**
     * Erzeugt eine neue Presentation auf dem angegebenen Picture, wobei alle
     * relevanten Einstellungen von dieser Presentation &uuml;bernommen werden.
     * Falls der PresentationType dieser Presentation nicht mit dem 
     * angegebenen Bild kompatibel ist, werde die DefaultPresentation
     * zur&uuml;ckgegeben.
     *
     * @param p Picture
     * @return Kopie von this passend zu p.
     */
    public Presentation getCopy(Picture p);

    /**
     * Erzeugt eine Zeichenkette, die Aufschluss ueber den PresentationType sowie
     * die aktuellen Einstellungen dieser Presentation geben. Der Ergebnisstring
     * habe die Form <tt>getPresentationType().getName() + "$" + ...</tt>, wobei
     * <tt>...</tt> fuer beliebige Zeichen steht, die die aktuellen Einstellungen
     * geeignet codieren.
     *
     * @return Parameterstring
     */
    public String toString();
}
