/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

/**
 * Ausf&uuml;hrlicher Kommentar: siehe Klassenkopf des Interfaces Presentation.
 *
 * @see Presentation
 */
public interface PresentationType {

    /**
     * Gibt den Namen des PresentationTypes zur&uuml;ck
     *
     * @return Name des PresentationTypes
     */
    public String getName();

    /**
     * Pr&uuml;ft, ob es m&ouml;glich ist, eine Presentation dieses Typs auf <tt>p</tt> 
     * zu erzeugen, ohne da&szlig; ein Fehler auftritt und liefert in diesem Fall true.
     * false wird z.B. zur&uuml;ckgegeben, wenn versucht wird, eine ColorPresentation
     * auf einem Bild mit nur zwei Layern zu erzeugen.
     *
     * @param p: Pr&uuml;fbild
     */
    public boolean isCompatible(Picture p);

    /**
     * Erzeugt eine Presentation dieses Types auf dem Bild <tt>p</tt>. 
     *
     * @exception PictureException, falls <tt>!this.isCompatible(p)</tt>
     */
    public Presentation createPresentation(Picture p) throws PictureException;

    /**
     * Erzeugt eine Presentation dieses Types auf dem Bild <tt>p</tt>
     * 
     * @exception PictureException, falls das Bild inkompatibel zu diesem Presentationtype ist.
     * @exception IllegalArgumentException, falls der Parameterstring ungueltig ist
     */
    public Presentation createPresentation(Picture p, String s) 
	throws PictureException, IllegalArgumentException;
}
