/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;

public class SelectPresentation implements PresentationType {

    private final PresentationType pt = this;

    private final class MyPresentation extends ColorModel implements Presentation {

	private final Picture p;
	private JRadioButtonMenuItem[] layers;

	public MyPresentation(Picture p) {

	    super(31); this.p = p;
	    layers = new JRadioButtonMenuItem[p.getNumberOfLayers()];
	    ButtonGroup b = new ButtonGroup();

	    for (int i = 0; i < p.getNumberOfLayers(); i++) {
		layers[i] = new JRadioButtonMenuItem("Ebene " + i);
		b.add(layers[i]);
	    }

	    layers[0].setSelected(true);
	}

	public MyPresentation(Picture p, String s)
	    throws IllegalArgumentException, PictureException {

	    this(p);
	    int i = new Integer(s).intValue();

	    if (i < 0 || i >= layers.length) 
		throw new PictureException("Ungltige Anzahl von Ebenen");
	    
	    layers[i].setSelected(true);
	}

	public int getRed(int pixel) {return pixel;}
	public int getBlue(int pixel) {return pixel;}
	public int getGreen(int pixel) {return pixel;}
	public int getAlpha(int pixel) {return 255;}

	public JPopupMenu getJPopupMenu(ActionListener al) {

	    JPopupMenu p = PresentationFactory.getJPopupMenu(this.p, al);
	    p.addSeparator();
	    for (int i = 0; i < layers.length; i++) {
		p.add(layers[i]);
		layers[i].addActionListener(al);
	    }

	    return p;
	}

	public Image getImage() {

	    Layer l = null;
	    for (int i = 0; i < layers.length; i++)
		if (layers[i].isSelected())
		    l = p.getLayer(i);

	    int width = l.getWidth();	    int height = l.getHeight();
	    double min = l.getMinimum();    double max = l.getMaximum();	    
	    final int cpp = 255;	    double m = cpp / (max - min);
	    int[] id = new int[width * height];

	    for (int x = 0; x < width; x++)
		for (int y = 0; y < height; y++)
		    id[y * width + x] = (int) ((l.data[x][y] - min) * m);

	    MemoryImageSource mis = new MemoryImageSource(width, height, this, id, 0, width);
	    return Toolkit.getDefaultToolkit().createImage(mis);
	}

	public PresentationType getPresentationType() {return pt;}

	public Presentation getCopy(Picture p) {

	    MyPresentation mp = new MyPresentation(p);
	    if (p.getNumberOfLayers() >= this.p.getNumberOfLayers()) 
		for (int i = 0; i < layers.length; i++)
		    if (layers[i].isSelected())
			mp.layers[i].setSelected(true);

	    return mp;
	}

	public String toString() {

	    int i;
	    for (i = 0; i < layers.length && !layers[i].isSelected(); i++);

	    return getName() + "$" + i;		
	}
    }

    public SelectPresentation() {}

    public boolean isCompatible(Picture mlp) {return true;}

    public String getName() {return "Auswahl-Darstellung";}

    public Presentation createPresentation(Picture p) {return new MyPresentation(p);}

    public Presentation createPresentation(Picture p, String s) 
	throws IllegalArgumentException, PictureException {
	
	return new MyPresentation(p, s);
    }
}
