/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filetype;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.util.*;
import java.io.*;

import picture.*;

/**
 * PictureType fr von Java bereitgestellte Dateiformate.
 */
public abstract class JavaImagePictureType extends PictureType {

    static {
	PictureTypeFactory.registerPictureType(new GIFPictureType());
	PictureTypeFactory.registerPictureType(new JPEGPictureType());
	PictureTypeFactory.registerPictureType(new PNGPictureType());
    }

    /**
     * Dummy zum Aufrufen des Static-Initializers ...
     */
    public static final void init() {
    }

    static class GIFPictureType extends JavaImagePictureType {
	public String getExtension() {
	    return "gif";
	}

	public String getDescription() {
	    return "Compuserve-GIF-Bild";
	}
    }

    static class JPEGPictureType extends JavaImagePictureType {
	public String getExtension() {
	    return "jpg";
	}

	public String getDescription() {
	    return "JPEG-Bild";
	}
    }

    static class PNGPictureType extends JavaImagePictureType {
	public String getExtension() {
	    return "png";
	}

	public String getDescription() {
	    return "PNG-Bild";
	}
    }

    public Picture load(InputStream origin) throws PictureException, IOException {

	try {

	    // Strategie:
	    // (1) Inhalt des Stroms in einen byte[],
	    // (2) daraus ein java.awt.Image erzeugen,
	    // (3) aus diesem dann data[][] bestuecken.
	    
	    // zu (1)
	    byte [] daten = new byte[origin.available()];
	    origin.read(daten);
	    
	    // zu (2)
	    Toolkit tk = Toolkit.getDefaultToolkit();
	    Image img = tk.createImage(daten);
	    tk.prepareImage(img, -1, -1, null);
	    
	    // zu (3)
	    while (img.getWidth(null) < 0 || img.getHeight(null) < 0) 
		Thread.sleep(5);

	    int width = img.getWidth(null);
	    int height = img.getHeight(null);
	    double[][][] rgb = new double[3][width][height];
	    int [] pixel = new int[width * height];
	    PixelGrabber pg = new PixelGrabber(img, 0, 0, width, height, pixel, 0, width);
	    pg.grabPixels();
	    boolean grau = true;
	    for (int i = 0; i < width; i++)
		for (int j = 0; j < height; j++) {
		    rgb[0][i][j] = (pixel[j * width + i] >> 16) & 0xff;
		    rgb[1][i][j] = (pixel[j * width + i] >>  8) & 0xff;
		    rgb[2][i][j] = (pixel[j * width + i]      ) & 0xff;
		    grau &= (rgb[0][i][j] == rgb[1][i][j] && rgb[1][i][j] == rgb[2][i][j]);
		}

	    Picture p;

	    if (grau) {
		p = new Picture(new Layer[]{new Layer(rgb[0])});
		p.setDefaultPresentation();
	    } else {
		p = new Picture(new Layer[]{new Layer(rgb[0]), 
					    new Layer(rgb[1]), new Layer(rgb[2])});
		p.setPresentation(PresentationFactory.presentationByName(p, "Farb-Darstellung"));
	    }
	    p.setPictureType(this);
	    return p;
		
	} catch (InterruptedException e) {
	    throw new PictureException("Fehler beim Laden des Bildes.");
	}
    }

    public void save(Picture p, PictureProperty pp, OutputStream out) 
	throws IOException, PictureException {

	throw new PictureException("Das Speichern wird noch nicht untersttzt.");
    }
}

