/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.filter;

import javax.swing.JFrame;

import picture.gui.*;
import picture.*;

/**
 * Diese Klasse implementiert Matrixfilter.
 *
 * Das sind Filter, bei denen der neue Wert eines Pixels durch die
 * umgebenden bestimmt wird, gewichtet durch die Matrix. Randpixel
 * werden dabei nicht bearbeitet. 
 */
public class MatrixFilter extends DefaultTransformation {

    protected double[][] matrix = {{1, 1, 1}, {1, 1, 1}, {1, 1, 1}};

    private double matrixSum = Double.NEGATIVE_INFINITY;

    private int mw = matrix.length / 2;
    private int mh = matrix[0].length / 2;

    public void setMatrix(double[][] matrix) throws PictureException {

	if(matrix.length < 1)
	    throw new PictureException("Matrix zu klein");
	if((matrix.length % 2) != 1 || (matrix[0].length % 2) != 1)
	    throw new PictureException("Matrix nicht ungerade");

	this.matrix = matrix;
	mw = matrix.length/2;
	mh = matrix[0].length/2;
	matrixSum = Double.NEGATIVE_INFINITY;
    }

    /**
     * Liefert die aktuelle Gre der Filtermatrix.
     */
    public int getSize() {
	return matrix.length;
    }

    public double getValue(int x, int y) {
	return matrix[x][y];
    }

    public void setValue(double c, int x, int y) {
	matrix[x][y] = c;
	matrixSum = Double.NEGATIVE_INFINITY;
    }

    private final double getMatrixFilteredPixel(Layer p, int x, int y) {

	if(matrixSum == Double.NEGATIVE_INFINITY) {
	    matrixSum = 0;
	    for(int i = 0; i < matrix.length; i++)
		for(int j = 0; j < matrix[0].length; j++)
		    matrixSum += Math.abs(matrix[i][j]);
	}

	double ps = 0;

	for(int i = x-mw; i <= x+mw; i++)
	    for(int j = y-mh; j <= y+mh; j++)
		ps += p.getValue(i, j) * matrix[i-x+mw][j-y+mw];

	return ps / matrixSum;
    }

    public int getNumberOfProgressSteps(Picture p) {

	return p.getWidth() * p.getNumberOfLayers();
    }
    
    public Layer of(ProgressHandler ph, Layer p) throws PictureException {

	int w = p.getWidth(), h = p.getHeight();
	double[][] data = new double[w][h];

	for(int x = 0; x < w; x++) {
	    for(int y = 0; y < h; y++)
		if((x >= mw) && (x < w-mw) && (y >= mh) && (y < h-mh))
		    data[x][y] = getMatrixFilteredPixel(p, x, y);
		else
		    data[x][y] = p.getValue(x, y);

	    if (ph != null) ph.step();
	}

	return createNewInstance(p, data);
    }

    /**
     * Liefert einen Optionendialog zurck.
     *
     * @param owner Parentframe
     * @param pic zu berbeitendes Bild
     * @return OptionenDialog
     */
    public DialogPane filterOptionPane(JFrame owner, Picture pic) 
	throws PictureException {

	return new MatrixPane(owner, pic, this);
    }    
}
