/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

/**
 * GUI-Komponente zur Manipulation eines Eigenwertes.
 */
public class EigenValue extends JPanel {

    private JSlider slider;
    private JCheckBox checkBox = new JCheckBox();
    private JLabel text = new JLabel("");
    private JComponent chart = null;
    private double lambda;
    private double max;
    private EigenValue next = null;
    private EventListener ev = new EventListener();
    private Vector changeListeners = new Vector();
    private boolean eastWest = true;

    private class EventListener extends MouseAdapter implements ItemListener, ChangeListener {

	// Doppelklicken auf den Slider bewirkt r"ucksetzen auf lambda
	public void mouseClicked(MouseEvent e) {
	    if (e.getClickCount() == 2)
		setValue(lambda);
	}

	// Slider bewegt
	public void stateChanged(ChangeEvent e) {

	    text.setText("" + (slider.getValue() * max / 100));
	    fireChangeEvent();
	}

	// Checkbox gedrueckt
	public void itemStateChanged(ItemEvent e) {
	    
	    setEnabled(checkBox.isSelected());
	    fireChangeEvent();
	}
    }

    public EigenValue(double val) {this(val, 2 * val);}

    public EigenValue(double val, double max) {this(val, max, null);}

    public EigenValue(double val, double max, JComponent chart) {

	this(val, max, chart, true);
    }

    public EigenValue(double val, double max, JComponent chart, boolean eastWest) {

	lambda = val; this.max = max; this.chart = chart; this.eastWest = eastWest;
	initGUI();
    }

    public EigenValue(double val, EigenValue next) {this(val, next, null, true);}

    public EigenValue(double val, EigenValue next, JComponent chart) {

	this(val, next, chart, true);
    }

    public EigenValue(double val, EigenValue next, JComponent chart, boolean eastWest) {

	this.next = next; lambda = val; max = next.max; this.chart = chart;
	this.eastWest = eastWest;
	initGUI();
    }

    public void setEnabled(boolean enable) {

	if (enable && !checkBox.isSelected()) return;

	if (next != null) next.setEnabledRadikal(enable);

	slider.setEnabled(enable);

	if (enable) 
	    text.setText("" + (slider.getValue() * max / 100));
	else
	    text.setText("0");
    }

    private void setEnabledRadikal(boolean enable) {

	setEnabled(enable);
	checkBox.setEnabled(enable);
	fireChangeEvent();
    }

    private void initGUI() {

	setLayout(new BorderLayout());
	
	slider = new JSlider(JSlider.HORIZONTAL, 0, 100, (int) (100 * (lambda / max)));
	checkBox.setSelected(true);
	text.setText("" + lambda);

	JPanel south = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0)); add("South", south);
	JPanel north = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0)); add("North", north);

	north.add(checkBox); north.add(slider);
	
	if (chart != null) south.add(chart);
	south.add(text);

	slider.addChangeListener(ev);
	slider.addMouseListener(ev);
	checkBox.addItemListener(ev);
    }

    private void setValue(double d) {

	slider.setValue((int) (d * 100 / max));
	text.setText("" + d);
    }

    public double getValue() {

	return (checkBox.isSelected() && slider.isEnabled()) ? 
	    (slider.getValue() * max / 100) : 0;
    }

    public void addChangeListener(ChangeListener cl) {

	changeListeners.addElement(cl);
    }

    private void fireChangeEvent() {

	Enumeration e = changeListeners.elements();
	ChangeEvent ev = new ChangeEvent(this);
	while (e.hasMoreElements()) 
	    ((ChangeListener)e.nextElement()).stateChanged(ev);
	    
    }

    public static void main(String [] args) {

	JFrame f = new JFrame("Test");
	try {
	    String win = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
	    UIManager.setLookAndFeel(win);
	    SwingUtilities.updateComponentTreeUI(f);
	} catch (Exception e) {e.printStackTrace();} 
	f.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	});

	JPanel p = new JPanel(new GridLayout(0, 1));
	EigenValue [] alt = new EigenValue[5];
	alt[4] = new EigenValue(Math.random() * 5, 10);
	for (int i = 3; i >= 0; i--)
	    alt[i] = new EigenValue(Math.random() * 5, alt[i + 1]);

	for (int i = 0; i < 5; i++)
	    p.add(alt[i]);

	f.getContentPane().add(p);
	f.pack();
	f.setVisible(true);
    } 
}
