/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.*;

import picture.*;
import picture.filter.*;
import picture.filetype.*;

/**
 * Savepane fuer das Abspeichern von Bildern im JIPP-Format
 */
public class JippSavePane extends DialogPane implements JippPictureProperty {

    // #############################################################################
    // ###  Codierungsauswaehler  ##################################################
    // #############################################################################

    private static final class CodingChooser extends JPanel implements ActionListener {

	private JList streams = new JList();
	private JList streams0 = new JList();
	Vector aktuell = new Vector();

	// Bei Aenderungen diese auch in JippSavePane.getCoding() durchfuehren!
	String [] data = new String[]{
	    "Burrows-Wheeler", "Move-To-Front", "RLE", 
	    "Huffman", "Arithmetisch", "LZ77/Huffman", "Differenz"
	};

	public CodingChooser () {

	    super(new BorderLayout(10, 10));

	    JPanel hinUndHer = new JPanel(new GridLayout(2, 1, 0, 10));

	    hinUndHer.add(getButton("hin")); hinUndHer.add(getButton("her"));
	    hinUndHer.setBorder(new EmptyBorder(10, 0, 10, 0));

	    add("West", getList(streams));
	    add("Center", hinUndHer);
	    add("East", getList(streams0));

	    streams0.setListData(data); 
	    streams0.setSelectedIndex(0);
	    streams0.setSelectionMode(0);
	    streams.setListData(aktuell); 
	    streams.setSelectedIndex(0);
	    streams.setSelectionMode(0);

	    aktuell.addElement(data[2]);
	    aktuell.addElement(data[0]);
	    aktuell.addElement(data[1]);
	    aktuell.addElement(data[2]);
	    aktuell.addElement(data[4]);
	}

	public Dimension getPreferredSize() {

	    return new Dimension(super.getPreferredSize().width, 100);
	}

	private JScrollPane getList(JList streams) {
	    
	    int v = JScrollPane.VERTICAL_SCROLLBAR_ALWAYS;
	    int h = JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED;
	    JScrollPane jsp = new JScrollPane(streams, v, h); 
	    jsp.setBorder(new BevelBorder(BevelBorder.LOWERED));
	    streams.setPrototypeCellValue("Burrows-Wheeler");
	    return jsp;
	}

	private JButton getButton(String s) {

	    Toolkit tk = Toolkit.getDefaultToolkit();
	    JButton b = new JButton(new ImageIcon(tk.getImage(GUIHelp.getImageURL(s + "Pfeil"))));
	    b.setActionCommand(s); b.addActionListener(this);
	    b.setMargin(new Insets(0, 10, 0, 10));
	    return b;
	}

	public void actionPerformed(ActionEvent e) {

	    String k = e.getActionCommand();
	    if (k.equals("hin")) {
		String v = (String) streams0.getSelectedValue();
		if (v != null) {
		    int sel = Math.min(aktuell.size() - 1, streams.getSelectedIndex());
		    aktuell.insertElementAt(v, sel + 1);
		    streams.setListData(aktuell);
		    streams.setSelectedIndex(sel + 1);
		}
	    } else if (k.equals("her")) 
		try {
		    int sel = streams.getSelectedIndex();
		    aktuell.removeElementAt(sel);
		    streams.setListData(aktuell);
		    streams.setSelectedIndex(Math.max(0, sel - 1));
		} catch (Exception warWohlNixMarkiert) {}

	    repaint();
	}
    }

    // #############################################################################
    // ###  EventListener  #########################################################
    // #############################################################################

    private class EventListener implements ActionListener, ChangeListener {

	public void actionPerformed(ActionEvent e) {

	    String k = e.getActionCommand();

	    if (k.equals("OK")) {
		closingCode = OK;
		dispose();
	    } else if (k.equals("Abbrechen")) {
		closingCode = ABBORD;
		dispose();
	    } else if (e.getSource() == grauwerte) {

		try {
		    long gw = new Long(grauwerte.getText()).longValue();
		    int bits = (int) (Math.log(gw) / Math.log(2));
		    if (bits > 0 && bits <= 64)
			bittiefe.setValue(bits);
		} catch (Exception ign) {}
	    } else if (e.getSource() == digitisieren) {

		bittiefe.setEnabled(!digitisieren.isSelected());
		grauwerte.setEnabled(!digitisieren.isSelected());
	    }
	}

	private boolean b = false;

	public void stateChanged(ChangeEvent e) {

	    Object s = e.getSource();

	    if (s == bittiefe) {
		String t = "" + (long) (Math.pow(2, bittiefe.getValue()));
		if (t.length() > 10) {
		    t = "ca. " + t.charAt(0) + "." + t.substring(1, 2) + "E" + t.length();
		}
		grauwerte.setText(t);
	    } else if (noiwdwa.getValue() > wavelet.getValue()) {
		if (s == noiwdwa) 
		    wavelet.setValue(noiwdwa.getValue());
		else if (s == wavelet) 
		    noiwdwa.setValue(wavelet.getValue());
	    } 
	} 
    }

    // #############################################################################
    // ###  Hauptklasse  ###########################################################
    // #############################################################################

    // ### Instanzvariablen ########################################################

    private JCheckBox md5 = new JCheckBox("MD5-Checksumme speichern", true);
    private JCheckBox details = new JCheckBox("Details anheben", true);
    private JCheckBox header = new JCheckBox("Header komprimieren", false);
    private JCheckBox digitisieren = new JCheckBox("doubles speichern", false);

    private JSlider wavelet = new JSlider(JSlider.HORIZONTAL);
    private JSlider noiwdwa = new JSlider(JSlider.HORIZONTAL);

    private JSlider bittiefe = new JSlider(JSlider.HORIZONTAL, 1, 63, 8);
    private JTextField grauwerte = new JTextField("256", 10);

    private CodingChooser coding = new CodingChooser();
    private JTextArea kommentar = new JTextArea(5, 15);

    private JButton ok = new JButton("OK");
    private JButton abbruch = new JButton("Abbrechen");

    private EventListener ev = new EventListener();

    // ### Konstruktor #############################################################

    public JippSavePane(JFrame owner, Picture pic) {

	super(owner, "JIPP-Dateiformat");

	JPanel c = new JPanel(new BorderLayout(0, 0));
	getContentPane().add(c);

	JPanel n = new JPanel(new BorderLayout(0, 0));
	JPanel s = new JPanel(new BorderLayout(0, 0));

	JPanel nw = new JPanel(new BorderLayout(0, 0));
	JPanel no = new JPanel(new BorderLayout(0, 0));
	JPanel sw = new JPanel(new GridLayout(1, 1, 0, 0));
	JPanel so = new JPanel(new GridLayout(1, 1, 0, 0));

	JPanel daten = new JPanel(new GridLayout(0, 1));
	JPanel aufl = new JPanel(new BorderLayout(0, 0));
	//JPanel quelle = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
	JPanel grau = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
	JPanel tiefe = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
	JPanel noOben = new JPanel(new GridLayout(0, 1, 0, 0));

	JPanel buttons = new JPanel(new GridLayout(1, 0, 10, 10));

	c.add("North", n); c.add("Center", s); c.add("South", buttons);
	n.add("West", nw); n.add("Center", no); s.add("West", sw); s.add("Center", so);

	nw.add("North", daten);	nw.add("South", aufl);

	//quelle.add(new JLabel("Quelle: ")); quelle.add(presentation);
	daten.setBorder(new EmptyBorder(5, 5, 5, 5));
	//daten.add(quelle); 
	daten.add(header); daten.add(md5);

	grau.add(grauwerte); grau.add(new JLabel(" Grauwerte"));
	tiefe.add(bittiefe); //tiefe.add(new JLabel("Bits pro\n Pixel"));

	aufl.setBorder(createBorder("Digitisierung"));
	aufl.add("North", digitisieren);
	aufl.add("Center", grau); aufl.add("South", tiefe);

	//noOben.add(waveletCompression);
	JPanel iterationPane = new JPanel(new BorderLayout(0, 0));
	iterationPane.add("North", new JLabel("Anzahl der Zerlegungsiterationen"));
	iterationPane.add("South", wavelet);
	JPanel noDetailsPane = new JPanel(new BorderLayout(0, 0));
	noDetailsPane.add("North", new JLabel("Iterationen ohne Detailspeicherung"));
	noDetailsPane.add("South", noiwdwa);

	no.setBorder(createBorder("Waveletoptionen"));
	no.add("North", details);
	no.add("Center", iterationPane);
	no.add("South", noDetailsPane);

	/*
	noOben.add(new JLabel("Anzahl der Zerlegungiterationen"));
	noOben.add(wavelet);
	//noOben.add(new JLabel("Anzahl detail-ignorierender"));
	noOben.add(new JLabel("Iterationen ohne Detailsspeicherung"));

	no.setBorder(createBorder("Waveletoptionen"));
	no.add("North", noOben);
	no.add("South", noiwdwa);
	*/

	sw.setBorder(createBorder("Codierungsverfahren"));
	sw.add(coding);

	JScrollPane jsp = new JScrollPane(kommentar, 
					  JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
					  JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	jsp.setBorder(new BevelBorder(BevelBorder.LOWERED));
	so.setBorder(createBorder("Kommentar"));
	so.add(jsp);

	buttons.add(ok); buttons.add(abbruch); buttons.add(new JPanel());
	buttons.setBorder(new EmptyBorder(10, 10, 10, 10));

	// Verkabelung
	ok.addActionListener(ev); ok.setActionCommand("OK");
	abbruch.addActionListener(ev); abbruch.setActionCommand("Abbrechen");

	grauwerte.addActionListener(ev);
	bittiefe.addChangeListener(ev);
	prepareSlider(bittiefe, 1, 62);
	bittiefe.setPaintTicks(false);
	Dictionary dic = bittiefe.getLabelTable();
	dic.put(new Integer(32), new JLabel(". . . Bits/Pixel . . ."));
	bittiefe.setLabelTable(dic);

	digitisieren.addActionListener(ev);

	//waveletCompression.addChangeListener(ev);
	int wavMax = 1;
	int w = pic.getWidth(); int h = pic.getHeight();
	if (w != h || !DefaultTransformation.isPowerOfTwo(w)) {
	    //waveletCompression.setEnabled(false);
	    wavelet.setEnabled(false);
	    noiwdwa.setEnabled(false);
	    details.setEnabled(false);
	} else 
	    wavMax = (int) (Math.log(w) / Math.log(2)) + 1;

	noiwdwa.setMinimum(0);	noiwdwa.setValue(0);	noiwdwa.setMaximum(wavMax);
	prepareSlider(noiwdwa, 1, 2); noiwdwa.addChangeListener(ev);
	wavelet.setMinimum(0);	wavelet.setValue(0);    wavelet.setMaximum(wavMax);
	prepareSlider(wavelet, 1, 2); wavelet.addChangeListener(ev);

	prepareWindow();
    }

    // ### Tools ###################################################################

    private Border createBorder(String title) {

	return new TitledBorder(BorderFactory.createEtchedBorder(), title);
    }

    private void prepareSlider(JSlider s, int min, int max) {

	s.setMinorTickSpacing(min);
	s.setMajorTickSpacing(max);
	s.setSnapToTicks(true);
	s.setPaintTicks(true);
	s.setPaintLabels(true);
    }

    // ### Implementierung der Schnittstellen ######################################

    public boolean waveletCompression() {return wavelet.getValue() > 0;}

    public boolean waveletDetailsScaling() {return details.isSelected();}

    public int getTotalNumberOfIterationsInWaveletAnalysis() {return wavelet.getValue();}

    public int getNumberOfIterationsWithoutDetailsInWaveletAnalysis() {
	
	return noiwdwa.getValue();
    }

    public Enumeration getCodingStreams() {

	Enumeration e = coding.aktuell.elements();
	Vector v = new Vector();
	while (e.hasMoreElements()) {
	    String s = (String) e.nextElement();
	    String q = "picture.io.";
	    if (s.equals("Burrows-Wheeler")) q += "BWT";
	    else if (s.equals("Move-To-Front")) q += "MTF";
	    else if (s.equals("RLE")) q += "RLE";
	    else if (s.equals("Arithmetisch")) q += "Arith";
	    else if (s.equals("Huffman")) q += "DefaultStreams$Huffman";
	    else if (s.equals("LZ77/Huffman")) q += "DefaultStreams$Deflate";
	    else if (s.equals("Differenz")) q += "Diff";
	    else System.err.println("Unknown Stream: " + s); 
	    v.insertElementAt(q, 0);	    
	}

	return v.elements();
    }

    public String[] getComment() {

	StringTokenizer st = new StringTokenizer(kommentar.getText(), "\n");
	Vector v = new Vector();
	while (st.hasMoreElements())
	    v.addElement(st.nextElement());

	String [] erg = new String[v.size()];
	for (int i = 0; i < v.size(); i++)
	    erg[i] = (String) v.elementAt(i);

	return erg;
    }

    public String getChecksum() {return md5.isSelected() ? "MD5" : null;}

    public boolean compressHeader() {return header.isSelected();}

    public boolean digitize() {return !digitisieren.isSelected();}

    public long getNumberOfGrayScales() {

	long l = 0;
	try {
	    l = new Long(grauwerte.getText()).longValue();
	    if (l <= 1)
		throw new Exception();
	} catch (Exception e) { 
	    // Falls im Textfeld keine Zahl angegeben ist (dann wirft der Longkonstruktor
	    // eine Exception) oder die angegebene Zahl negativ ist (dann werfe ich selbst
	    // eine Exception, s.o.) wird der Wert aus dem Schieberegler genommen. Dieser
	    // kann keine ungueltigen Daten enthalten.
	    l = (long) (Math.pow(2, bittiefe.getValue()));
	}
	return l - 1;
    }

    // ### Testprogramm ############################################################

    public static void main(String [] args) {

	JFrame f = new JFrame();
	JippSavePane j = new JippSavePane(f, new Picture());
	do {
	    j.show();
	    Enumeration e = j.getCodingStreams();
	    while (e.hasMoreElements())
		System.out.println((String) e.nextElement());
	    System.out.println();
	} while (j.getClosingCode() == j.OK);

	System.exit(0);
    }
}
