/*
 * Copyright (c) 2000, 2001 Manuel Kauers, Stefan Rhrich
 * Alle Rechte vorbehalten.
 */

package picture.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import picture.*;
import picture.filter.*;

/**
 * Diese Klasse erlaubt die Einstellungen zur Matrixfilterung.
 */
public class MatrixPane extends DialogPane {

    private MatrixFilter matrix;

    private class MatrixTableModel extends AbstractTableModel {
	private int size;

	MatrixTableModel() {
	    size = matrix.getSize();
	}
	public int getSize() {
	    return size;
	}

	public void setSize(int size) {

	    double[][] m = new double[size][size];
	    this.size = size;
	    for(int i = 0; i < size; i++)
		for(int j = 0; j < size; j++)
		    m[i][j] = 1;
	    try {
		matrix.setMatrix(m);
	    } catch(PictureException e) {
		GUIHelp.error(e);
	    }
	    fireTableStructureChanged();
	}

	public int getColumnCount() {
	    return size;
	}
	
	public int getRowCount() {
	    return size;
	}

	public Object getValueAt(int row, int col) {
	    return new Double(matrix.getValue(col, row));
	}

	public void setValueAt(Object c, int row, int col) {
	    matrix.setValue(new Complex(c.toString()).getRealPart(), col, row);
	    fireTableCellUpdated(row, col);
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
	    return true;
	}
    }
    
    private JSlider groesse = new JSlider(JSlider.HORIZONTAL, 1, 9, 3);

    MatrixTableModel dataModel;
    JTable table;
    JScrollPane scrollpane;
    TableColumnModel tcm;

    /**
     * Erzeugt ein neues MatrixPane
     */
    public MatrixPane(Frame owner, Picture pic) throws PictureException {
	this(owner, pic, new MatrixFilter());
    }

    /**
     * Erzeugt ein neues MatrixPane aus einer gegebenen Matrix.
     */
    public MatrixPane(Frame owner, Picture pic, MatrixFilter mf) 
	throws PictureException {

	super(owner, "Matrixfilter", pic);

	matrix = mf;

	dataModel = new MatrixTableModel();
	table = new JTable(dataModel);
	scrollpane = new JScrollPane(table);
	tcm = table.getColumnModel();


        groesse.setMajorTickSpacing(2);
        groesse.setSnapToTicks(true);
        groesse.setPaintTicks(true);
        groesse.setPaintLabels(true);
	groesse.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
		    int t;

		    if((dataModel.getSize() != (t = groesse.getValue())) && (t > 0) && ((t%2) == 1)) {
			dataModel.setSize(groesse.getValue());
			for(int i = 0; i < tcm.getColumnCount(); i++)
			    tcm.getColumn(i).setPreferredWidth(100);
		    }
		}
	    });

	table.setTableHeader(null);
	table.setPreferredScrollableViewportSize(new Dimension(300, 100));
	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

	for(int i = 0; i < tcm.getColumnCount(); i++)
	    tcm.getColumn(i).setPreferredWidth(100);

	dataModel.addTableModelListener(new TableModelListener() {
		public void tableChanged(TableModelEvent e) {
		    updater.add(matrix);
		}
	    });

	/*
	 *   Erzeugung der Komponenten
	 */

	Container mitte = getContentPane();

	mitte.add(scrollpane, BorderLayout.CENTER);
	mitte.add(groesse, BorderLayout.SOUTH);

	/*
	 *  Verkabelung
	 */

	prepareWindow();
	updater.add(matrix);
    }

    /** Gibt die Transformation aus den aktuellen Einstellungen zurueck */
    public Transformation getTransformation() {
	return matrix;
    }
}
